/*
 * Copyright (C) 2010 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/delay.h>
#include "armadillo400_iomux.h"
#include "armadillo400_gpio.h"

#define mxc_iomux_v3a_setup_multiple_pads(PAD) \
	mxc_iomux_v3_setup_multiple_pads(PAD, ARRAY_SIZE(PAD))

static struct pad_desc armadillo400_ext_gpio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_21_GPIO1_8)
	MX25_PAD_CSI_MCLK__GPIO_1_8(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_GPIO1_9)
	MX25_PAD_CSI_VSYNC__GPIO_1_9(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_GPIO1_10)
	MX25_PAD_CSI_HSYNC__GPIO_1_10(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_GPIO1_11)
	MX25_PAD_CSI_PIXCLK__GPIO_1_11(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_25_GPIO1_16)
	MX25_PAD_CSPI1_SS0__GPIO_1_16(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_26_GPIO)
	MX25_PAD_CSPI1_RDY__GPIO_2_22(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#elif defined(CONFIG_ARMADILLO400_CON9_26_W1)
	MX25_PAD_CSPI1_RDY__GPIO_2_22(PAD_CTL_PKE | PAD_CTL_PUS_22K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_27_GPIO2_21)
	MX25_PAD_CLKO__GPIO_2_21(0), /* no pull up */
#endif
#if defined(CONFIG_ARMADILLO400_CON9_28_GPIO3_15)
	MX25_PAD_EXT_ARMCLK__GPIO_3_15(), /* no pull up */
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_GPIO1_17)
	MX25_PAD_CSPI1_SS1__GPIO_1_17(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_GPIO1_29)
	MX25_PAD_CSI_D4__GPIO_1_29(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_GPIO1_18)
	MX25_PAD_CSPI1_SCLK__GPIO_1_18(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_GPIO1_30)
	MX25_PAD_CSI_D5__GPIO_1_30(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_GPIO1_7)
	MX25_PAD_CSI_D8__GPIO_1_7(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_16_GPIO1_31)
	MX25_PAD_CSI_D6__GPIO_1_31(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_GPIO4_21)
	MX25_PAD_CSI_D9__GPIO_4_21(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_18_GPIO1_6)
	MX25_PAD_CSI_D7__GPIO_1_6(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_1_GPIO3_17) || defined(CONFIG_ARMADILLO400_CON9_1_SDHC2_PWREN)
	MX25_PAD_VSTBY_REQ__GPIO_3_17(PAD_CTL_PKE),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_2_GPIO3_14)
	MX25_PAD_RTCK__GPIO_3_14(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_GPIO_2_31)
	MX25_PAD_KPP_ROW2__GPIO_2_31(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_GPIO_3_0)
	MX25_PAD_KPP_ROW3__GPIO_3_0(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_44_GPIO_3_1)
	MX25_PAD_KPP_COL0__GPIO_3_1(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_GPIO_3_2)
	MX25_PAD_KPP_COL1__GPIO_3_2(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_GPIO_3_3)
	MX25_PAD_KPP_COL2__GPIO_3_3(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_GPIO_3_4)
	MX25_PAD_KPP_COL3__GPIO_3_4(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_48_GPIO1_0)
	MX25_PAD_GPIO_A__GPIO_A(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_GPIO1_1)
	MX25_PAD_GPIO_B__GPIO_B(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_3_GPIO1_2)
	MX25_PAD_GPIO_C__GPIO_C(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_GPIO1_3)
	MX25_PAD_GPIO_D__GPIO_D(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_3_GPIO1_14)
	MX25_PAD_CSPI1_MOSI__GPIO_1_14(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_4_GPIO1_27)
	MX25_PAD_CSI_D2__GPIO_1_27(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_GPIO1_15)
	MX25_PAD_CSPI1_MISO__GPIO_1_15(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_GPIO1_28)
	MX25_PAD_CSI_D3__GPIO_1_28(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

void gpio_common_active(void)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_ext_gpio_pads);
}
EXPORT_SYMBOL(gpio_common_active);

/*
 * Audio
 */
static struct pad_desc armadillo400_audio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_44_AUD5_TXD)
	MX25_PAD_KPP_COL0__AUD5_TXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_DOWN),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_AUD5_RXD)
	MX25_PAD_KPP_COL1__AUD5_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_AUD5_TXC)
	MX25_PAD_KPP_COL2__AUD5_TXC(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_AUD5_TXFS)
	MX25_PAD_KPP_COL3__AUD5_TXFS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_AUD5_RXC)
	MX25_PAD_KPP_ROW2__AUD5_RXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_AUD5_RXFS)
	MX25_PAD_KPP_ROW3__AUD5_RXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif

#if defined(CONFIG_ARMADILLO400_CON9_21_AUD6_TXD)
	MX25_PAD_CSI_MCLK__AUD6_TXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_DOWN),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_AUD6_RXD)
	MX25_PAD_CSI_VSYNC__AUD6_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_AUD6_TXC)
	MX25_PAD_CSI_HSYNC__AUD6_TXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_AUD6_TXFS)
	MX25_PAD_CSI_PIXCLK__AUD6_TXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_15_AUD6_RXC)
	MX25_PAD_CSI_D8__AUD6_RXC(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_17_AUD6_RXFS)
	MX25_PAD_CSI_D9__AUD6_RXFS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

void gpio_activate_audio_ports(void)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_audio_pads);
}
EXPORT_SYMBOL(gpio_activate_audio_ports);

/*
 * FEC
 */
static struct pad_desc armadillo400_fec_pads[] = {
	MX25_PAD_FEC_TX_CLK__FEC_TX_CLK(PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_FEC_RX_DV__FEC_RX_DV(PAD_CTL_PUE),
	MX25_PAD_FEC_RDATA0__FEC_RDATA0(PAD_CTL_PUE),
	MX25_PAD_FEC_TDATA0__FEC_TDATA0(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_FEC_TX_EN__FEC_TX_EN(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_FEC_MDC__FEC_MDC(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_FEC_MDIO__FEC_MDIO(PAD_CTL_HYS | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP),
	MX25_PAD_FEC_RDATA1__FEC_RDATA1(PAD_CTL_PUE),
	MX25_PAD_FEC_TDATA1__FEC_TDATA1(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_CTL_GRP_DSE_FEC(PAD_CTL_DSE_LOW),

	/* PHY_RST */
	MX25_PAD_VSTBY_ACK__GPIO_3_18(0),
	/* LINK_LED */
	MX25_PAD_UPLL_BYPCLK__GPIO_3_16(),
};

void gpio_fec_active(void)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_fec_pads);
}
EXPORT_SYMBOL(gpio_fec_active);

/*
 * I2C
 */
static struct pad_desc armadillo400_i2c1_pads[] = {
	MX25_PAD_I2C1_CLK__I2C1_CLK(PAD_CTL_PUS_100K_UP | PAD_CTL_ODE),
	MX25_PAD_I2C1_DAT__I2C1_DAT(PAD_CTL_PUS_100K_UP | PAD_CTL_ODE),
};

static struct pad_desc armadillo400_i2c1_gpio_pads[] = {
	MX25_PAD_I2C1_CLK__GPIO_1_12(PAD_CTL_PUS_100K_UP),
	MX25_PAD_I2C1_DAT__GPIO_1_13(PAD_CTL_PUS_100K_UP),
};

static struct pad_desc __maybe_unused armadillo400_i2c2_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON14_3_I2C2_SCL)
	MX25_PAD_GPIO_C__I2C2_SCL(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_I2C2_SDA)
	MX25_PAD_GPIO_D__I2C2_SDA(PAD_CTL_PKE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
};

static struct pad_desc __maybe_unused armadillo400_i2c2_gpio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON14_3_I2C2_SCL)
	MX25_PAD_GPIO_C__GPIO_C(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
#if defined(CONFIG_ARMADILLO400_CON14_4_I2C2_SDA)
	MX25_PAD_GPIO_D__GPIO_D(PAD_CTL_PKE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
};

static struct pad_desc __maybe_unused armadillo400_i2c3_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_48_I2C3_SCL)
	MX25_PAD_GPIO_A__I2C3_SCL(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_I2C3_SDA)
	MX25_PAD_GPIO_B__I2C3_SDA(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP | PAD_CTL_ODE),
#endif
};

static struct pad_desc __maybe_unused armadillo400_i2c3_gpio_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_48_I2C3_SCL)
	MX25_PAD_GPIO_A__GPIO_A(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_49_I2C3_SDA)
	MX25_PAD_GPIO_B__GPIO_B(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_22K_UP),
#endif
};

#define GPIO_I2C_DUMMY_CLOCK_NUM	128
#define GPIO_I2C_DUMMY_CLOCK_FREQ	40000

static int gpio_i2c_dummy_clock(unsigned gpio_clk, unsigned gpio_dat)
{
	int i;

	if (gpio_request(gpio_clk, "i2c_clk_gpio"))
		return -EINVAL;

	if (gpio_request(gpio_dat, "i2c_dat_gpio")) {
		gpio_free(gpio_clk);
		return -EINVAL;
	}

	gpio_direction_output(gpio_clk, 0);
	gpio_direction_input(gpio_dat);

	if (gpio_get_value(gpio_dat) == 0) {
		for (i = 0; i < GPIO_I2C_DUMMY_CLOCK_NUM; i++) {
			gpio_set_value(gpio_clk, 1);
			udelay(1000*1000/GPIO_I2C_DUMMY_CLOCK_FREQ);
			gpio_set_value(gpio_clk, 0);
			udelay(1000*1000/GPIO_I2C_DUMMY_CLOCK_FREQ);
		}
	}

	gpio_free(gpio_clk);
	gpio_free(gpio_dat);

	return 0;
}

void gpio_i2c_active(int i2c_num)
{
	switch (i2c_num) {
	case 0:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c1_gpio_pads);
		gpio_i2c_dummy_clock(I2C1_CLK_GPIO, I2C1_DAT_GPIO);
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c1_pads);
		break;
	case 1:
#if defined(CONFIG_ARMADILLO400_I2C2_CON14)
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c2_gpio_pads);
		gpio_i2c_dummy_clock(I2C2_CLK_GPIO, I2C2_DAT_GPIO);
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c2_pads);
#endif
		break;
	case 2:
#if defined(CONFIG_ARMADILLO400_I2C3_CON11)
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c3_gpio_pads);
		gpio_i2c_dummy_clock(I2C3_CLK_GPIO, I2C3_DAT_GPIO);
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_i2c3_pads);
#endif
		break;
	default:
		break;
	}
}
EXPORT_SYMBOL(gpio_i2c_active);

void gpio_i2c_inactive(int i2c_num)
{
}
EXPORT_SYMBOL(gpio_i2c_inactive);

/*
 * KEY
 */
static struct pad_desc armadillo400_gpio_keys_pads[] = {
	/* SW1 */
	MX25_PAD_NFWP_B__GPIO_3_30(),
#if defined(CONFIG_MACH_ARMADILLO440)
	/* EXT_GPIO18 */
	MX25_PAD_DE_B__GPIO_2_20(0), /* 47K_UP is always on */
	/* EXT_GPIO19 */
#if defined(CONFIG_ARMADILLO400_CON11_40_GPIO_2_29)
	MX25_PAD_KPP_ROW0__GPIO_2_29(PAD_CTL_PKE | PAD_CTL_PUS_47K_UP),
#endif
	/* EXT_GPIO20 */
#if defined(CONFIG_ARMADILLO400_CON11_41_GPIO_2_30)
	MX25_PAD_KPP_ROW1__GPIO_2_30(PAD_CTL_PKE | PAD_CTL_PUS_47K_UP),
#endif
#endif
};

void gpio_key_active(void)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_gpio_keys_pads);
}
EXPORT_SYMBOL(gpio_key_active);

/*
 * LCD
 */
static struct pad_desc armadillo400_lcdc_pads[] = {
	MX25_PAD_LD0__LD0(0),
	MX25_PAD_LD1__LD1(0),
	MX25_PAD_LD2__LD2(0),
	MX25_PAD_LD3__LD3(0),
	MX25_PAD_LD4__LD4(0),
	MX25_PAD_LD5__LD5(0),
	MX25_PAD_LD6__LD6(0),
	MX25_PAD_LD7__LD7(0),
	MX25_PAD_LD8__LD8(0),
	MX25_PAD_LD9__LD9(0),
	MX25_PAD_LD10__LD10(0),
	MX25_PAD_LD11__LD11(0),
	MX25_PAD_LD12__LD12(0),
	MX25_PAD_LD13__LD13(0),
	MX25_PAD_LD14__LD14(0),
	MX25_PAD_LD15__LD15(0),
	MX25_PAD_GPIO_E__LD16(0),
	MX25_PAD_GPIO_F__LD17(0),
	MX25_PAD_HSYNC__HSYNC(0),
	MX25_PAD_VSYNC__VSYNC(0),
	MX25_PAD_LSCLK__LSCLK(PAD_CTL_SRE_FAST),
	MX25_PAD_OE_ACD__OE_ACD(0),
	MX25_PAD_CTL_GRP_DSE_LCD(PAD_CTL_DSE_LOW),
};

void gpio_lcdc_active(void)
{
	mxc_iomux_v3a_setup_multiple_pads(armadillo400_lcdc_pads);
}
EXPORT_SYMBOL(gpio_lcdc_active);

void gpio_lcdc_inactive(void)
{
}
EXPORT_SYMBOL(gpio_lcdc_inactive);

void board_power_lcd(int on)
{
}
EXPORT_SYMBOL(board_power_lcd);

/*
 * PWM
 */
static struct pad_desc armadillo400_pwm1_pads[] = {
	MX25_PAD_PWM__PWM(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
};

static struct pad_desc armadillo400_pwm2_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_25_PWMO2)
	MX25_PAD_CSPI1_SS0__PWM2(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

static struct pad_desc armadillo400_pwm3_pads[] = {
};

static struct pad_desc armadillo400_pwm4_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON14_3_PWMO4)
	MX25_PAD_GPIO_C__PWM4(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
};

void gpio_activate_pwm(int id)
{
	switch (id) {
	case 0:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_pwm1_pads);
		break;
	case 1:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_pwm2_pads);
		break;
	case 2:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_pwm3_pads);
		break;
	case 3:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_pwm4_pads);
		break;
	default:
		/* This ID is not supported */
		break;
	}
}
EXPORT_SYMBOL(gpio_activate_pwm);

/*
 * SDHC
 */
static struct pad_desc __maybe_unused armadillo400_sdhc1_pads[] = {
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
	MX25_PAD_SD1_CMD__SD1_CMD(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_SD1_CLK__SD1_CLK(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_SD1_DATA0__SD1_DATA0(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_SD1_DATA1__SD1_DATA1(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_SD1_DATA2__SD1_DATA2(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),
	MX25_PAD_SD1_DATA3__SD1_DATA3(PAD_CTL_PUE | PAD_CTL_PUS_100K_UP | PAD_CTL_SRE_FAST),

	MX25_PAD_CTL_GRP_DSE_SDHC1(PAD_CTL_DSE_HIGH),

	/* SDHC1_CD */
	MX25_PAD_NFRB__GPIO_3_31(0),
	/* SDHC1_PWREN */
	MX25_PAD_NFRE_B__GPIO_3_27(),
#endif
};

static struct pad_desc __maybe_unused armadillo400_sdhc2_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_16_SDHC2_CMD)
	MX25_PAD_CSI_D6__SD2_CMD(PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_18_SDHC2_CLK)
	MX25_PAD_CSI_D7__SD2_CLK(PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_21_SDHC2_DATA0)
	MX25_PAD_CSI_MCLK__SD2_DATA0(PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_22_SDHC2_DATA1)
	MX25_PAD_CSI_VSYNC__SD2_DATA1(PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_23_SDHC2_DATA2)
	MX25_PAD_CSI_HSYNC__SD2_DATA2(PAD_CTL_SRE_FAST),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_24_SDHC2_DATA3)
	MX25_PAD_CSI_PIXCLK__SD2_DATA3(PAD_CTL_HYS | PAD_CTL_SRE_FAST),
#endif

	MX25_PAD_CTL_GRP_DSE_CSI(PAD_CTL_DSE_HIGH),

	/* SDHC2_CD */
#if defined(CONFIG_ARMADILLO400_CON9_17_SDHC2_CD)
	MX25_PAD_CSI_D9__GPIO_4_21(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
	/* SDHC2_WP */
#if defined(CONFIG_ARMADILLO400_CON9_15_SDHC2_WP)
	MX25_PAD_CSI_D8__GPIO_1_7(PAD_CTL_HYS | PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
	/* SDHC2_PWREN */
#if defined(CONFIG_ARMADILLO400_CON9_1_SDHC2_PWREN)
	MX25_PAD_VSTBY_REQ__GPIO_3_17(PAD_CTL_PKE),
#endif
};

static void gpio_sdhc_request(int port)
{
	static int requested = 0;

	if (requested & (1 << port))
		return;

	switch (port) {
	case 0:
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
		gpio_request(SDHC1_CLK_GPIO, "SDHC1_CLK");
		gpio_request(SDHC1_CD_GPIO, "SDHC1_CD");
#endif
		break;
	case 1:
#if defined(CONFIG_ARMADILLO400_SDHC2_CON9)
		gpio_request(SDHC2_CLK_GPIO, "SDHC2_CLK");
		gpio_request(SDHC2_CD_GPIO, "SDHC2_CD");
		gpio_request(SDHC2_WP_GPIO, "SDHC2_WP");
#endif
		break;
	default:
		break;
	}

	requested |= (1 << port);
}

void gpio_sdhc_active(int port)
{
	gpio_sdhc_request(port);

	switch (port) {
	case 0:
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_sdhc1_pads);

		/* Card-Detect pin */
		gpio_direction_input(SDHC1_CD_GPIO);
#endif
		break;
	case 1:
#if defined(CONFIG_ARMADILLO400_SDHC2_CON9)
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_sdhc2_pads);

		/* Card-Detect pin */
		gpio_direction_input(SDHC2_CD_GPIO);

		/* Write-Protect pin */
		gpio_direction_input(SDHC2_WP_GPIO);
#endif
		break;
	default:
		/* This ID is not supported */
		break;
	}
}
EXPORT_SYMBOL(gpio_sdhc_active);

void gpio_sdhc_inactive(int port)
{
	gpio_sdhc_request(port);

	switch (port) {
	case 0:
#if defined(CONFIG_ARMADILLO400_SDHC1_CON1)
	{
		struct pad_desc sdhc1_inactive[] = {
			MX25_PAD_SD1_CLK__GPIO_2_24(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_DOWN),
		};
		mxc_iomux_v3a_setup_multiple_pads(sdhc1_inactive);

		/* ENGcm02759: See "Chip Errata for the i.MX25" for details. */
		gpio_direction_output(SDHC1_CLK_GPIO, 0);

		gpio_direction_input(SDHC1_CD_GPIO);
	}
#endif
		break;
	case 1:
#if defined(CONFIG_ARMADILLO400_SDHC2_CON9)
	{
		struct pad_desc sdhc2_inactive[] = {
			MX25_PAD_CSI_D7__GPIO_1_6(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_DOWN),
		};
		mxc_iomux_v3a_setup_multiple_pads(sdhc2_inactive);

		/* ENGcm02759: See "Chip Errata for the i.MX25" for details. */
		gpio_direction_output(SDHC2_CLK_GPIO, 0);

		gpio_direction_input(SDHC2_CD_GPIO);
		gpio_direction_input(SDHC2_WP_GPIO);
	}
#endif
		break;
	default:
		/* This ID is not supported */
		break;
	}
}
EXPORT_SYMBOL(gpio_sdhc_inactive);

/*
 * UART
 */
static struct pad_desc armadillo400_uart_pad =
	MX25_PAD_CTL_GRP_DSE_UART(PAD_CTL_DSE_LOW);

static struct pad_desc armadillo400_uart2_pads[] = {
	MX25_PAD_UART2_RXD__UART2_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART2_TXD__UART2_TXD(0),
	MX25_PAD_UART2_RTS__UART2_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART2_CTS__UART2_CTS(0),
	MX25_PAD_UART1_RXD__UART2_DTR(0),
	MX25_PAD_UART1_TXD__UART2_DSR(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART1_RTS__UART2_DCD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	MX25_PAD_UART1_CTS__UART2_RI(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
	/* UART2_FORCEOFF */
	MX25_PAD_BOOT_MODE1__GPIO_4_31(),
};

static struct pad_desc armadillo400_uart3_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_3_UART3_RXD)
	MX25_PAD_CSPI1_MOSI__UART3_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_40_UART3_RXD)
	MX25_PAD_KPP_ROW0__UART3_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_5_UART3_TXD)
	MX25_PAD_CSPI1_MISO__UART3_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_41_UART3_TXD)
	MX25_PAD_KPP_ROW1__UART3_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_11_UART3_RTS)
	MX25_PAD_CSPI1_SS1__UART3_RTS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_42_UART3_RTS)
	MX25_PAD_KPP_ROW2__UART3_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_13_UART3_CTS)
	MX25_PAD_CSPI1_SCLK__UART3_CTS(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_43_UART3_CTS)
	MX25_PAD_KPP_ROW3__UART3_CTS(0),
#endif
};

static struct pad_desc armadillo400_uart4_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON11_44_UART4_RXD)
	MX25_PAD_KPP_COL0__UART4_RXD(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_45_UART4_TXD)
	MX25_PAD_KPP_COL1__UART4_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_46_UART4_RTS)
	MX25_PAD_KPP_COL2__UART4_RTS(PAD_CTL_PKE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON11_47_UART4_CTS)
	MX25_PAD_KPP_COL3__UART4_CTS(0),
#endif
};

static struct pad_desc armadillo400_uart5_pads[] = {
#if defined(CONFIG_ARMADILLO400_CON9_4_UART5_RXD)
	MX25_PAD_CSI_D2__UART5_RXD(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_6_UART5_TXD)
	MX25_PAD_CSI_D3__UART5_TXD(0),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_12_UART5_RTS)
	MX25_PAD_CSI_D4__UART5_RTS(PAD_CTL_PKE | PAD_CTL_PUE | PAD_CTL_PUS_100K_UP),
#endif
#if defined(CONFIG_ARMADILLO400_CON9_14_UART5_CTS)
	MX25_PAD_CSI_D5__UART5_CTS(0),
#endif
};

static int gpio_uart_activated = 0;
void gpio_uart_active(int port, int no_irda)
{
	mxc_iomux_v3_setup_pad(&armadillo400_uart_pad);

	switch (port) {
	case 0:
		break;
	case 1:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart2_pads);
		if (!(gpio_uart_activated & (1 << port)))
			gpio_request(UART2_FORCEOFF_GPIO, "UART2_FORCEOFF");
		gpio_direction_output(UART2_FORCEOFF_GPIO, 1);
		break;
	case 2:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart3_pads);
		break;
	case 3:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart4_pads);
		break;
	case 4:
		mxc_iomux_v3a_setup_multiple_pads(armadillo400_uart5_pads);
		break;
	default:
		break;
	}

	gpio_uart_activated |= (1 << port);
}
EXPORT_SYMBOL(gpio_uart_active);

void gpio_uart_inactive(int port, int no_irda)
{
	switch (port) {
	case 0:
		break;
	case 1:
		gpio_set_value(UART2_FORCEOFF_GPIO, 0);
		break;
	case 2:
		break;
	case 3:
		break;
	case 4:
		break;
	default:
		break;
	}
}
EXPORT_SYMBOL(gpio_uart_inactive);

void config_uartdma_event(int port)
{
}
EXPORT_SYMBOL(config_uartdma_event);

/*
 * USB
 */
static struct pad_desc armadillo400_usb_pads[] = {
	MX25_PAD_NFWE_B__GPIO_3_26(),
};
static int usb_gpio_use_count = 0;

static int gpio_usb_request(void)
{
	if (!usb_gpio_use_count) {
		mxc_iomux_v3_setup_multiple_pads(armadillo400_usb_pads,
						 ARRAY_SIZE(armadillo400_usb_pads));

		gpio_request(USB_PWRSEL_GPIO, "USB_PWRSEL");
		gpio_direction_output(USB_PWRSEL_GPIO, USB_PWRSRC_5V);
	}
	usb_gpio_use_count++;

	return 0;
}

static void usb_gpio_free(void)
{
	if (usb_gpio_use_count > 0 && !(--usb_gpio_use_count)) {
		gpio_free(USB_PWRSEL_GPIO);
	}
}

int gpio_usbh2_active(void)
{
	int ret;

	ret = gpio_usb_request();
	if (ret != 0)
		return ret;

	return 0;
}
EXPORT_SYMBOL(gpio_usbh2_active);

void gpio_usbh2_inactive(void)
{
	usb_gpio_free();
}
EXPORT_SYMBOL(gpio_usbh2_inactive);

int gpio_usbotg_utmi_active(void)
{
	int ret;

	ret = gpio_usb_request();
	if (ret != 0)
		return ret;

	return 0;
}
EXPORT_SYMBOL(gpio_usbotg_utmi_active);

void gpio_usbotg_utmi_inactive(void)
{
	usb_gpio_free();
}
EXPORT_SYMBOL(gpio_usbotg_utmi_inactive);
